@@ -5,13 +5,15 @@
 #include "domain_layers.h"
 #include "domain_records.h"
 #include "domain_search.h"
+#include "public.h"
 #include "handle_consumer_ops.h"
+#include "expirables.h"
 #include "db_log.h"
 
 static struct event_base *event_base;
 
-void http_close_cb(struct evhttp_connection * const cnx,
-                   void * const context_)
+static void http_close_cb(struct evhttp_connection * const cnx,
+                          void * const context_)
 {
     (void) context_;
     evhttp_connection_set_closecb(cnx, NULL, NULL);
@@ -33,8 +35,8 @@ int send_op_reply(HttpHandlerContext * const context,
             break;
         }
         ts = (struct timespec) {
-            .tv_sec = tv.tv_sec + 1,
-            .tv_nsec = tv.tv_usec * 1000
+            .tv_sec = tv.tv_sec + 1L,
+            .tv_nsec = tv.tv_usec * 1000L
         };
         pthread_mutex_lock(&fake_mutex);
         pthread_cond_timedwait(&fake_cond, &fake_mutex, &ts);
@@ -85,7 +87,7 @@ stab:
         } else if (op.bare_op.type == OP_TYPE_LAYERS_CREATE) {
             pthread_rwlock_wrlock(&context->rwlock_layers);
             ret = handle_op_layers_create(&op.layers_create_op, context);
-                pthread_rwlock_unlock(&context->rwlock_layers);
+            pthread_rwlock_unlock(&context->rwlock_layers);
         } else if (op.bare_op.type == OP_TYPE_LAYERS_DELETE) {
             pthread_rwlock_wrlock(&context->rwlock_layers);
             ret = handle_op_layers_delete(&op.layers_delete_op, context);
@@ -126,6 +128,14 @@ stab:
 #endif
             ret = handle_op_search_in_rect(&op.search_in_rect_op, context);
             pthread_rwlock_unlock(&context->rwlock_layers);
+        } else if (op.bare_op.type == OP_TYPE_SEARCH_IN_KEYS) {
+#if AUTOMATICALLY_CREATE_LAYERS
+            pthread_rwlock_wrlock(&context->rwlock_layers);
+#else
+            pthread_rwlock_rdlock(&context->rwlock_layers);
+#endif
+            ret = handle_op_search_in_keys(&op.search_in_keys_op, context);
+            pthread_rwlock_unlock(&context->rwlock_layers);
         } else {
             assert(0);
         }
@@ -153,7 +163,8 @@ stab:
                             (unsigned int) sizeof "error" - (size_t) 1U);
             yajl_gen_string(json_gen, (const unsigned char *) "error",
                             (unsigned int) sizeof "error" - (size_t) 1U);
-            send_op_reply(context, op_reply);                
+            send_op_reply(context, op_reply);
+            yajl_gen_free(json_gen);
         }
     } else {
         pthread_cond_wait(&context->cond_cqueue, &context->mtx_cqueue);
@@ -162,46 +173,53 @@ stab:
     return 0;
 }
 
-void *worker_thread(void *context_)
+static void *worker_thread(void *context_)
 {
     HttpHandlerContext * const context = context_;
-    printf("Worker thread: [%p]\n", (void *) pthread_self());
+    uint32_t thr_id;
+    
+    evutil_secure_rng_get_bytes(&thr_id, sizeof thr_id);
+    logfile(context, LOG_INFO,
+            "Starting worker thread: [%" PRIu32 "]", thr_id);
     
     while (worker_do_work(context) == 0);
     
-    printf("Exited worker thread: [%p]\n", (void *) pthread_self());
+    logfile(context, LOG_INFO, "Exited worker thread: [%" PRIu32 "]", thr_id);
     
     return NULL;
 }
 
-int process_request(HttpHandlerContext *context,
-                    struct evhttp_request * const req,
-                    const _Bool fake_req)
+static char *extract_opts(char * const decoded_uri)
 {
-    const char *uri;    
-    char *decoded_uri;
-    size_t decoded_uri_len;
-    const char *ext = ".json";
-    const size_t ext_len = strlen(ext);
-    char *domain;
-    char *pnt;
-    
-    uri = evhttp_request_get_uri(req);
-    if (strncmp(uri, context->encoded_base_uri,
-                context->encoded_base_uri_len) != 0) {
-        if (fake_req == 0) {
-            evhttp_send_error(req, HTTP_NOTFOUND, "Not Found");
-        }
-        return -1;
-    }
-    decoded_uri = evhttp_decode_uri(uri + context->encoded_base_uri_len);
     char * opts;
+    
     if ((opts = strchr(decoded_uri, '?')) != NULL) {
         *opts = 0;
         if (*++opts == 0) {
             opts = NULL;
         }
     }
+    return opts;
+}
+
+static int process_api_request(HttpHandlerContext * const context,
+                               const char * const uri,
+                               struct evhttp_request * const req,
+                               const _Bool fake_req)
+{
+    char *decoded_uri;
+    size_t decoded_uri_len;
+    const char *ext = ".json";
+    const size_t ext_len = strlen(ext);
+    char *domain;
+    char *pnt;
+
+    if (fake_req == 0) {
+        evhttp_add_header(req->output_headers, "Content-Type",
+                          "application/json; charset=UTF-8");    
+    }
+    decoded_uri = evhttp_decode_uri(uri + context->encoded_api_base_uri_len);
+    char * const opts = extract_opts(decoded_uri);
     decoded_uri_len = strlen(decoded_uri);
     if (decoded_uri_len <= ext_len || decoded_uri_len > MAX_URI_LEN ||
         strcasecmp(decoded_uri + decoded_uri_len - ext_len, ext) != 0) {
@@ -248,24 +266,56 @@ int process_request(HttpHandlerContext *context,
         if (fake_req == 0) {        
             evhttp_send_error(req, ret, "Error");
         }
-        return- 1;
+        return -1;
     }
     if (fake_req == 0 && write_to_log != 0) {
-        add_to_db_log(req->type, uri, evhttp_request_get_input_buffer(req));
+        add_to_db_log(context->now, req->type, uri,
+                      evhttp_request_get_input_buffer(req));
     }
-    return 0;
+    return 0;    
 }
 
-void http_dispatcher_cb(struct evhttp_request * const req,
-                        void * const context_)
+static int process_public_request(HttpHandlerContext * const context,
+                                  const char * const uri,
+                                  struct evhttp_request * const req)
+{
+    char *decoded_uri;
+    decoded_uri = evhttp_decode_uri(uri + context->encoded_public_base_uri_len);
+    char * const opts = extract_opts(decoded_uri);
+    const int ret = handle_public_request(context, decoded_uri, opts, req);
+    free(decoded_uri);
+    
+    return ret;
+}
+
+static int process_request(HttpHandlerContext * const context,
+                           struct evhttp_request * const req,
+                           const _Bool fake_req)
+{
+    const char * const uri = evhttp_request_get_uri(req);
+    if (strncmp(uri, context->encoded_api_base_uri,
+                context->encoded_api_base_uri_len) == 0) {
+        return process_api_request(context, uri, req, fake_req);
+    }
+    if (strncmp(uri, context->encoded_public_base_uri,
+                context->encoded_public_base_uri_len) == 0) {
+        assert(fake_req == 0);
+        return process_public_request(context, uri, req);
+    }
+    if (fake_req == 0) {
+        evhttp_send_error(req, HTTP_NOTFOUND, "Not Found");
+    }
+    return -1;
+}
+
+static void http_dispatcher_cb(struct evhttp_request * const req,
+                               void * const context_)
 {
     HttpHandlerContext *context = context_;
     
     evhttp_connection_set_closecb(evhttp_request_get_connection(req),
                                   http_close_cb, req);
     evhttp_add_header(req->output_headers, "X-Server", SERVER_NAME);
-    evhttp_add_header(req->output_headers, "Content-Type",
-                      "application/json; charset=UTF-8");
 #ifdef NO_KEEPALIVE
     evhttp_add_header(req->output_headers, "Connection", "close");
 #endif
@@ -294,7 +344,7 @@ int fake_request(HttpHandlerContext * const context,
     return 0;
 }
 
-void free_layer_slab_entry_cb(void *layer_)
+static void free_layer_slab_entry_cb(void *layer_)
 {
     Layer * const layer = layer_;
     free(layer->name);
@@ -302,9 +352,22 @@ void free_layer_slab_entry_cb(void *layer_)
     free_pan_db(&layer->pan_db);
 }
 
+static void free_expirables_slab_entry_cb(void *expirable_)
+{
+    Expirable * const expirable = expirable_;
+    *expirable = (Expirable) {
+        .ts = (time_t) 0,
+        .key_node = NULL
+    };
+}
+
 static RETSIGTYPE sigterm_cb(const int sig)
 {
     (void) sig;
+    if (app_context.db_log.journal_rewrite_process != (pid_t) -1) {
+        kill(app_context.db_log.journal_rewrite_process, SIGKILL);
+        app_context.db_log.journal_rewrite_process = (pid_t) -1;
+    }
     if (event_base != NULL) {
         event_base_loopbreak(event_base);
     }
@@ -409,11 +472,79 @@ static void flush_log_db(evutil_socket_t fd, short event,
     flush_db_log(1);
     struct timeval tv = {
         .tv_sec = app_context.db_log.fsync_period,
-        .tv_usec = 0
+        .tv_usec = 0L
     };
     evtimer_add(&context->ev_flush_log_db, &tv);
 }
 
+static void expiration_cron(evutil_socket_t fd, short event,
+                            void * const context_)
+{
+    HttpHandlerContext * const context = context_;
+    
+    (void) event;
+    (void) fd;
+    if (time(&context->now) == (time_t) -1) {
+        return;
+    }
+    struct timeval tv = {
+        .tv_sec = 1L,
+        .tv_usec = 0L
+    };
+#if SPREAD_EXPIRATION
+    const int ret = purge_expired_keys(context);
+    if (ret != 0 && context->nb_layers > (size_t) 1U) {
+        const long required_usec = 1000000L / (long) context->nb_layers;
+        if (required_usec < 10000L) {
+            while (purge_expired_keys(context) > 0);
+        } else {
+            tv = (struct timeval) {
+                .tv_sec = 0L,
+                .tv_usec = required_usec
+            };
+        }
+    }
+#else
+    purge_expired_keys(context);
+#endif
+    evtimer_add(&context->ev_expiration_cron, &tv);
+}
+
+static int open_log_file(HttpHandlerContext * const context)
+{
+    int flags = O_RDWR | O_CREAT | O_APPEND;
+#ifdef O_EXLOCK
+    flags |= O_EXLOCK;
+#endif
+#ifdef O_NOATIME
+    flags |= O_NOATIME;
+#endif
+#ifdef O_LARGEFILE
+    flags |= O_LARGEFILE;
+#endif
+    assert(context->log_fd == -1);
+    if (app_context.log_file_name == NULL) {
+        return 0;
+    }
+    context->log_fd = open(context->log_file_name, flags, (mode_t) 0600);
+    if (context->log_fd == -1) {
+        logfile(NULL, LOG_ERR, "Can't open [%s]: [%s]", context->log_file_name,
+                strerror(errno));
+        return -1;
+    }
+    return 0;
+}
+
+static int close_log_file(HttpHandlerContext * const context)
+{
+    if (context->log_fd != -1) {
+        fsync(context->log_fd);
+        close(context->log_fd);
+        context->log_fd = -1;
+    }
+    return 0;
+}
+
 int http_server(void)
 {
     HttpHandlerContext http_handler_context = {
@@ -421,11 +552,24 @@ int http_server(void)
         .thr_workers = NULL,
         .event_base = NULL,
         .op_tid = (OpTID) 0U,
-        .encoded_base_uri = NULL,
+        .encoded_api_base_uri = NULL,
+        .encoded_public_base_uri = NULL,
         .cqueue = NULL,
         .publisher_bev = NULL,
-        .consumer_bev = NULL
+        .consumer_bev = NULL,
+        .nb_layers = (size_t) 0U,
+        .log_file_name = NULL,
+        .log_fd = -1
     };
+    if (time(&http_handler_context.now) == (time_t) -1) {
+        return -1;
+    }
+    http_handler_context.log_file_name = app_context.log_file_name;    
+    if (open_log_file(&http_handler_context) != 0) {
+        return -1;
+    }
+    logfile_noformat(&http_handler_context, LOG_INFO,
+                     PACKAGE_STRING " started.");
     struct evhttp *event_http;
     struct bufferevent *bev_pair[2];
     set_signals();
@@ -433,6 +577,10 @@ int http_server(void)
                   sizeof(Layer), "layers") != 0) {
         return -1;
     }
+    if (init_slab(&http_handler_context.expirables_slab,
+                  sizeof(Expirable), "expirables") != 0) {
+        return -1;
+    }
     http_handler_context.cqueue = malloc(sizeof *http_handler_context.cqueue);
     if (http_handler_context.cqueue == NULL ||
         init_cqueue(http_handler_context.cqueue,
@@ -442,9 +590,12 @@ int http_server(void)
     pthread_mutex_init(&http_handler_context.mtx_cqueue, NULL);
     pthread_cond_init(&http_handler_context.cond_cqueue, NULL);
     pthread_rwlock_init(&http_handler_context.rwlock_layers, NULL);
-    http_handler_context.encoded_base_uri = ENCODED_BASE_URI;
-    http_handler_context.encoded_base_uri_len =
-        strlen(http_handler_context.encoded_base_uri);
+    http_handler_context.encoded_api_base_uri = ENCODED_API_BASE_URI;
+    http_handler_context.encoded_api_base_uri_len =
+        strlen(http_handler_context.encoded_api_base_uri);
+    http_handler_context.encoded_public_base_uri = ENCODED_PUBLIC_BASE_URI;
+    http_handler_context.encoded_public_base_uri_len =
+        strlen(http_handler_context.encoded_public_base_uri);
     evthread_use_pthreads();
     event_base = event_base_new();
     http_handler_context.event_base = event_base;
@@ -474,19 +625,29 @@ int http_server(void)
                        flush_log_db, &http_handler_context);
         struct timeval tv = {
             .tv_sec = app_context.db_log.fsync_period,
-            .tv_usec = 0
+            .tv_usec = 0L
         };
         evtimer_add(&http_handler_context.ev_flush_log_db, &tv);
     }
     replay_log(&http_handler_context);
     if (start_workers(&http_handler_context) != 0) {
         goto bye;
-    }    
+    }
+    evtimer_assign(&http_handler_context.ev_expiration_cron, event_base,
+                   expiration_cron, &http_handler_context);
+    struct timeval tv = {
+        .tv_sec = 0L,
+        .tv_usec = 0L
+    };
+    evtimer_add(&http_handler_context.ev_expiration_cron, &tv);
+    
     event_base_dispatch(event_base);
+    
     stop_workers(&http_handler_context);
     if (app_context.db_log.fsync_period > 0) {
         evtimer_del(&http_handler_context.ev_flush_log_db);
     }
+    evtimer_del(&http_handler_context.ev_expiration_cron);
 bye:
     bufferevent_free(http_handler_context.publisher_bev);
     bufferevent_free(http_handler_context.consumer_bev);
@@ -497,6 +658,9 @@ bye:
     pthread_mutex_destroy(&http_handler_context.mtx_cqueue);
     pthread_rwlock_destroy(&http_handler_context.rwlock_layers);
     free_slab(&http_handler_context.layers_slab, free_layer_slab_entry_cb);
+    free_slab(&http_handler_context.expirables_slab,
+              free_expirables_slab_entry_cb);
+    close_log_file(&http_handler_context);
     
     return 0;
 }
