@@ -110,8 +110,9 @@ typedef struct RecordsGetOp_ {
     struct evhttp_request *req;
     _Bool fake_req;    
     OpTID op_tid;
-    Key *layer_name;    
-    Key *key;    
+    Key *layer_name;
+    Key *key;
+    _Bool with_links;
 } RecordsGetOp;
 
 typedef struct RecordsDeleteOp_ {
@@ -134,6 +135,7 @@ typedef struct SearchNearbyOp_ {
     SubSlots limit;
     Dimension epsilon;
     _Bool with_properties;
+    _Bool with_links;    
 } SearchNearbyOp;
 
 typedef struct SearchInRectOp_ {
@@ -146,6 +148,7 @@ typedef struct SearchInRectOp_ {
     SubSlots limit;
     Dimension epsilon;
     _Bool with_properties;
+    _Bool with_links;    
 } SearchInRectOp;
 
 typedef struct SearchInKeysOp_ {
@@ -158,6 +161,7 @@ typedef struct SearchInKeysOp_ {
     SubSlots limit;
     _Bool with_properties;
     _Bool with_content;
+    _Bool with_links;    
 } SearchInKeysOp;
 
 typedef union Op_ {
