@@ -1,5 +1,6 @@
 
 #include "common.h"
+#include "key_nodes.h"
 #include "utils.h"
 
 void skip_spaces(const char * * const str)
@@ -11,8 +12,36 @@ void skip_spaces(const char * * const str)
     *str = s;
 }
 
+typedef struct TraversedKeyNode_ {
+    KeyNode * key_node;
+    _Bool traversed;
+} TraversedKeyNode;
+
+typedef struct TraversalStackIsDuplicateCBContext_ {
+    const void * const linked_key_s;
+    const size_t linked_key_s_len;
+    _Bool found;
+} TraversalStackIsDuplicateCBContext;
+
+static int traversal_stack_is_duplicate_cb(void * const context_,
+                                           void * const traversed_key_node_)
+{
+    TraversalStackIsDuplicateCBContext * const context = context_;
+    const TraversedKeyNode * const traversed_key_node = traversed_key_node_;
+    const Key * const traversed_key = traversed_key_node->key_node->key;
+    
+    if (strcmp(context->linked_key_s, traversed_key->val) == 0) {
+        assert(context->linked_key_s_len == traversed_key->len);
+        context->found = 1;
+        return 1;
+    }    
+    return 0;
+}
+
 typedef struct RecordsGetPropertiesCBContext_ {
-    yajl_gen json_gen;    
+    yajl_gen json_gen;
+    PanDB * const pan_db;
+    PntStack * const traversal_stack;
 } RecordsGetPropertiesCBContext;
 
 static int records_get_properties_cb(void * const context_,
@@ -22,17 +51,63 @@ static int records_get_properties_cb(void * const context_,
                                      const size_t value_len)
 {
     RecordsGetPropertiesCBContext * const context = context_;
-    
+
     yajl_gen_string(context->json_gen, (const unsigned char *) key,
-                    (unsigned int) key_len);
+                    (unsigned int) key_len);    
     yajl_gen_string(context->json_gen, (const unsigned char *) value,
-                    (unsigned int) value_len);    
+                    (unsigned int) value_len);
+    if (context->traversal_stack == NULL ||
+        key_len <= sizeof INT_PROPERTY_LINK - (size_t) 1U ||
+        memcmp(key, INT_PROPERTY_LINK,
+               sizeof INT_PROPERTY_LINK - (size_t) 1U) != 0) {
+        return 0;
+    }
+    if (value_len <= (size_t) 0U || (* (const char *) value) == 0) {
+        return 0;
+    }
+    int status;
+    KeyNode *linked_key_node;
+    Key * const linked_key = new_key_with_leading_zero(value, value_len);
+    status = get_key_node_from_key(context->pan_db, linked_key, 0,
+                                   &linked_key_node);
+    if (status < 0) {
+        release_key(linked_key);        
+        return -1;
+    }
+    if (status == 0) {
+        release_key(linked_key);        
+        return 0;
+    }
+    TraversalStackIsDuplicateCBContext
+        traversal_stack_is_duplicate_cb_context = {
+            .linked_key_s = linked_key->val,
+            .linked_key_s_len = linked_key->len,
+            .found = 0
+        };
+    pnt_stack_foreach(context->traversal_stack,
+                      traversal_stack_is_duplicate_cb,
+                      &traversal_stack_is_duplicate_cb_context);
+    release_key(linked_key);    
+    if (traversal_stack_is_duplicate_cb_context.found != 0) {
+        return 0;
+    }
+    TraversedKeyNode traversed_key_node = {
+        .key_node = linked_key_node,
+        .traversed = 0
+    };
+    if (push_pnt_stack(context->traversal_stack, &traversed_key_node) != 0) {
+        return -1;
+    }
     return 0;
 }
 
-int key_node_to_json(KeyNode * const key_node, yajl_gen json_gen,
-                     const _Bool with_properties)
+static int key_node_to_json_(KeyNode * const key_node, yajl_gen json_gen,
+                             PanDB * const pan_db,                             
+                             const _Bool with_properties,
+                             PntStack * const traversal_stack)
 {
+    (void) traversal_stack;
+    
     yajl_gen_string(json_gen, (const unsigned char *) "key",
                     (unsigned int) sizeof "key" - (size_t) 1U);
     yajl_gen_string(json_gen, (const unsigned char *) key_node->key->val,
@@ -91,7 +166,9 @@ int key_node_to_json(KeyNode * const key_node, yajl_gen json_gen,
         yajl_gen_integer(json_gen, (long) key_node->expirable->ts);
     }
     RecordsGetPropertiesCBContext cb_context = {
-        .json_gen = json_gen
+        .json_gen = json_gen,
+        .pan_db = pan_db,
+        .traversal_stack = traversal_stack
     };
     if (key_node->properties != NULL) {
         yajl_gen_string(json_gen, (const unsigned char *) "properties",
@@ -104,6 +181,86 @@ int key_node_to_json(KeyNode * const key_node, yajl_gen json_gen,
     return 0;
 }
 
+typedef struct TraverseKeysNodesForeachCBContext_ {
+    yajl_gen json_gen;
+    PanDB * const pan_db;
+    _Bool with_properties;
+    PntStack * const traversal_stack;
+    _Bool found_job;
+} TraverseKeysNodesForeachCBContext;
+
+static int traverse_keys_nodes_foreach_cb(void * const context_,
+                                          void * const traversed_key_node_)
+{
+    TraverseKeysNodesForeachCBContext * const context = context_;
+    TraversedKeyNode * const traversed_key_node = traversed_key_node_;
+    
+    if (traversed_key_node->traversed == 1) {
+        return 0;
+    }
+    const Key * const key = traversed_key_node->key_node->key;
+    yajl_gen_string(context->json_gen,
+                    (const unsigned char *) key->val,
+                    (unsigned int) (key->len - (size_t) 1U));
+    yajl_gen_map_open(context->json_gen);
+    const int ret = key_node_to_json_(traversed_key_node->key_node,
+                                      context->json_gen, context->pan_db,
+                                      context->with_properties,
+                                      context->traversal_stack);
+    yajl_gen_map_close(context->json_gen);
+    traversed_key_node->traversed = 1;
+    context->found_job = 1;
+    
+    return ret;
+}
+
+int key_node_to_json(KeyNode * const key_node, yajl_gen json_gen,
+                     PanDB * const pan_db,
+                     const _Bool with_properties,
+                     const _Bool with_links)
+{
+    if (with_links == 0) {    
+        return key_node_to_json_(key_node, json_gen, pan_db,
+                                 with_properties, NULL);
+    }    
+    PntStack * const traversal_stack =
+        new_pnt_stack(INITIAL_TRAVERSAL_STACK_SIZE, sizeof(TraversedKeyNode));
+    if (traversal_stack == NULL) {
+        return -1;
+    }
+    TraversedKeyNode traversed_key_node = {
+        .key_node = key_node,
+        .traversed = 1
+    };    
+    if (push_pnt_stack(traversal_stack, &traversed_key_node) != 0) {
+        free_pnt_stack(traversal_stack);
+        return -1;
+    }
+    if (key_node_to_json_(key_node, json_gen, pan_db, with_properties,
+                          traversal_stack) != 0) {
+        free_pnt_stack(traversal_stack);
+        return -1;
+    }
+    TraverseKeysNodesForeachCBContext context = {
+        .json_gen = json_gen,
+        .pan_db = pan_db,
+        .with_properties = with_properties,
+        .traversal_stack = traversal_stack
+    };
+    yajl_gen_string(json_gen, (const unsigned char *) "$links",
+                    (unsigned int) sizeof "$links" - (size_t) 1U);
+    yajl_gen_map_open(json_gen);
+    do {
+        context.found_job = 0;
+        pnt_stack_foreach(traversal_stack, traverse_keys_nodes_foreach_cb,
+                          &context);
+    } while (context.found_job != 0);
+    yajl_gen_map_close(json_gen);
+    free_pnt_stack(traversal_stack);
+    
+    return 0;
+}
+
 // Thanks to the CompeGPS team for their help!
 
 Position2D sin_projection(const Position2D * const position)
@@ -224,8 +381,8 @@ Meters fast_distance_between_geoidal_positions(const Position2D * const p1,
     return (Meters) d;
 }
 
-Meters romboid_distance_between_geoidal_positions(const Position2D * const p1,
-                                                  const Position2D * const p2)
+Meters rhomboid_distance_between_geoidal_positions(const Position2D * const p1,
+                                                   const Position2D * const p2)
 {
     const float k = cosf(DEG_TO_RAD(p1->latitude));
     const float dx = k * (p1->longitude - p2->longitude);
